@@ -138,7 +138,9 @@ class IndexedArray(awkward.array.base.AwkwardArrayWithContent):
 
             self._isvalid = True
 
-    def __iter__(self):
+    def __iter__(self, checkiter=True):
+        if checkiter:
+            self._checkiter()
         self._valid()
         for i in self._index:
             yield self._content[i]
@@ -303,7 +305,9 @@ class ByteIndexedArray(IndexedArray):
 
             self._isvalid = True
 
-    def __iter__(self):
+    def __iter__(self, checkiter=True):
+        if checkiter:
+            self._checkiter()
         self._valid()
         itemsize = self._dtype.itemsize
         for i in self._index:
@@ -530,7 +534,9 @@ class SparseArray(awkward.array.base.AwkwardArrayWithContent):
 
             self._isvalid = True
 
-    def __iter__(self):
+    def __iter__(self, checkiter=True):
+        if checkiter:
+            self._checkiter()
         self._valid()
 
         length = self._length
